==> libpqxx-1.2.2.tar.gz: libpqxx-1.2.2/test/test41.cxx <==
#include <cassert>
#include <iostream>

#include <pqxx/cachedresult.h>
#include <pqxx/connection.h>
#include <pqxx/transaction.h>
#include <pqxx/result.h>

using namespace PGSTD;
using namespace pqxx;

namespace
{

// Verify that CachedResult::at() catches an index overrun
void CheckOverrun(const CachedResult &CR, CachedResult::size_type Overrun)
{
  if (Overrun >= 0) Overrun += CR.size();

  bool OK = false;
  try
  {
    CR.at(Overrun).at(0);
  }
  catch (const exception &e)
  {
    // OK, this is what we expected to happen
    OK = true;
    cerr << "(Expected) " << e.what() << endl;
  }

  if (!OK) 
    throw logic_error("Failed to detect overrun "
	              "(" + ToString(Overrun) + " rows)");
}

}


// Test program for libpqxx.  Compare behaviour of a CachedResult to a regular
// Result.
//
// Usage: test41 [connect-string]
//
// Where connect-string is a set of connection options in Postgresql's
// PQconnectdb() format, eg. "dbname=template1" to select from a database
// called template1, or "host=foo.bar.net user=smith" to connect to a
// backend running on host foo.bar.net, logging in as user smith.
int main(int, char *argv[])
{
  try
  {
    Connection C(argv[1] ? argv[1] : "");
    Transaction T(C, "test41");

    const char Query[] = "SELECT * FROM pg_tables";

    Result R( T.Exec(Query) );

    for (int BlockSize = 2; BlockSize <= R.size()+1; ++BlockSize)
    {
      CachedResult CR(T, Query, "cachedresult", BlockSize);
 
      // Verify that we get an exception if we exceed CR's range, and are able
      // to recover afterwards
      for (CachedResult::size_type n = -5; n < 5; ++n) CheckOverrun(CR, n);

      // Compare contents for CR with R
      for (Result::size_type i = R.size() - 1; i >= 0 ; --i)
      {
	string A, B;
	R.at(i).at(0).to(A);
	CR.at(i).at(0).to(B);
	if (A != B)
	  throw logic_error("BlockSize " + ToString(BlockSize) + ", "
	                    "row " + ToString(i) + ": "
			    "Expected '" + A + "', "
			    "got '" + B + "'");
      }
    }
  }
  catch (const exception &e)
  {
    // All exceptions thrown by libpqxx are derived from std::exception
    cerr << "Exception: " << e.what() << endl;
    return 2;
  }
  catch (...)
  {
    // This is really unexpected (see above)
    cerr << "Unhandled exception" << endl;
    return 100;
  }

  return 0;
}

==> libpqxx-1.2.8.tar.gz: libpqxx-1.2.8/test/test41.cxx <==
#include <cassert>
#include <iostream>

#include <pqxx/cachedresult.h>
#include <pqxx/connection.h>
#include <pqxx/transaction.h>
#include <pqxx/result.h>

using namespace PGSTD;
using namespace pqxx;

namespace
{

// Verify that CachedResult::at() catches an index overrun
void CheckOverrun(const CachedResult &CR, CachedResult::size_type Overrun)
{
  if (Overrun >= 0) Overrun += CR.size();

  bool OK = false;
  try
  {
    CR.at(Overrun).at(0);
  }
  catch (const exception &e)
  {
    // OK, this is what we expected to happen
    OK = true;
    cerr << "(Expected) " << e.what() << endl;
  }

  if (!OK) 
    throw logic_error("Failed to detect overrun "
	              "(" + ToString(Overrun) + " rows)");
}

}


// Test program for libpqxx.  Compare behaviour of a CachedResult to a regular
// Result.
//
// Usage: test41 [connect-string]
//
// Where connect-string is a set of connection options in Postgresql's
// PQconnectdb() format, eg. "dbname=template1" to select from a database
// called template1, or "host=foo.bar.net user=smith" to connect to a
// backend running on host foo.bar.net, logging in as user smith.
int main(int, char *argv[])
{
  try
  {
    Connection C(argv[1] ? argv[1] : "");
    Transaction T(C, "test41");

    const char Query[] = "SELECT * FROM events";

    Result R( T.Exec(Query) );

    for (int BlockSize = 2; BlockSize <= R.size()+1; ++BlockSize)
    {
      CachedResult CR(T, Query, "cachedresult", BlockSize);
 
      // Verify that we get an exception if we exceed CR's range, and are able
      // to recover afterwards
      for (CachedResult::size_type n = -5; n < 5; ++n) CheckOverrun(CR, n);

      // Compare contents for CR with R
      for (Result::size_type i = R.size() - 1; i >= 0 ; --i)
      {
	string A, B;
	R.at(i).at(0).to(A);
	CR.at(i).at(0).to(B);
	if (A != B)
	  throw logic_error("BlockSize " + ToString(BlockSize) + ", "
	                    "row " + ToString(i) + ": "
			    "Expected '" + A + "', "
			    "got '" + B + "'");
      }
    }
  }
  catch (const exception &e)
  {
    // All exceptions thrown by libpqxx are derived from std::exception
    cerr << "Exception: " << e.what() << endl;
    return 2;
  }
  catch (...)
  {
    // This is really unexpected (see above)
    cerr << "Unhandled exception" << endl;
    return 100;
  }

  return 0;
}
